/* Версия: V2 от 30.11.2023
   Железо: ESP8266 (Wemos D1 mini) + RS485 to TTL converter (MAX485)
   Подключение: 5(D1) ESP <-> DI TTL
                0(D3) ESP <-> RO TTL
                4(D2) ESP <-> RE TTL
                GND ESP <-> GND TTL
                5V ESP <-> VCC TTL
   Мониторинг параметров и статуса контроллера. Передача на сервис аналитики http://open-monitoring.online
   Статья с подробным описанием:
   Автор: Nikolay Semerkov, https://nsk-electro.ru/
   Для связи с автором: semerkov@mail.ru
   В версии 2 исправлена ошибка с отображением отрицательной температуры контроллера
*/

// Строки требующие редактирования в комментариях отмечены (!)

#include <ESP8266WiFi.h>                          // Стандартная библиотека в пакете
#include <SoftwareSerial.h>                       // Стандартная библиотека в пакете 
#include <ModbusMaster.h>                         // Скачать библиотеку https://github.com/4-20ma/ModbusMaster
#define FIRMWARE_VER 2                            // Версия

//*-- Настройки точки доступа
const char* ssid     = "XXXX";                    // Название точки доступа     (!)
const char* password = "XXXX";                    // Пароль                     (!)

//*-- Параметры полученные при регистрации на http://open-monitoring.online
const char* host = "open-monitoring.online";      // Адрес сервера (не менять)
const char* streamId   = "XXX";                   // ID                         (!)
const char* privateKey = "XXXXXX";                // Код доступа                (!)
unsigned long send_interval = 61000;              // Периодичность отправки пакетов на сервер (не менее 60 сек)
unsigned long request_interval = 15000;           // Периодичность опроса контроллера, желат. 3-4 опроса между отправками
unsigned long previousMillis = 0;
unsigned long requestMillis = 0;
float previousEnergy;

#define MAX485_DE      4                          // Пины для подключения RE и DE (4 - для платы ОМ)             
#define MAX485_RE_NEG  4
#define RX_PIN 0                                  // Rx - программного порта
#define TX_PIN 5                                  // Tx - программного порта
SoftwareSerial softSerial;

// instantiate ModbusMaster object
ModbusMaster node;

// Переменные физ. величин контроллера
//      1     2      3      4       5       6       7         9           10            11        12-15 - резерв
float pv_V, pv_I, batt_V, batt_I, load_U, load_I, batt_T, EnergyToday, EnergyTotal, deltaEnergy;
//      8
int batt_SOC;
//         16           17      19-20 резерв
String batt_Status, ch_Status;

uint8_t result;
uint16_t data[6];

boolean flag_1Send = true;    // Флаг первой посылки
boolean flag_TotEn = false;   // Флаг получения значения полной энергии
boolean flag_startEn = true;  // Флаг начального значения энергии
boolean LedStatus = false;    // флаг светодиода
boolean dataTrade = false;    // флаг состояния обмена данными


//*---------------- INICIALIZATION ------------------*//
void setup() {
  pinMode(MAX485_RE_NEG, OUTPUT);
  pinMode(MAX485_DE, OUTPUT);
  pinMode(2, OUTPUT);
  // Init in receive mode
  digitalWrite(MAX485_RE_NEG, 0);
  digitalWrite(MAX485_DE, 0);

  Serial.begin(115200);
  softSerial.begin(115200, SWSERIAL_8N1, RX_PIN, TX_PIN, false, 32, 192);
  softSerial.enableIntTx(false);
  delay(10);
  Serial.println("");
  Serial.println("");
  Serial.print("Версия ПО: ");
  Serial.println(FIRMWARE_VER);

  // Адрес контроллера в сети ModBus (по-умолчанию - 1)
  node.begin(1, softSerial);
  // Callbacks allow us to configure the RS485 transceiver correctly
  node.preTransmission(preTransmission);
  node.postTransmission(postTransmission);

  // Подключение к сети WiFi
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    digitalWrite(2, LedStatus);
    LedStatus = !LedStatus;      // Мигаем каждые 0,5 сек пока идет подключение к сети WiFi
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}


//*------------------ MAIN CYCLE --------------------*//
void loop() {
  
  if ( millis() - requestMillis >= request_interval  || flag_1Send == true) {
    requestMillis = millis();
    if ( dataTrade == true ) {
      digitalWrite(2, false);     // Если связь есть - зажигаем светодиод на время запроса в солн. контроллер
    }
    else {
      digitalWrite(2, true);      // Если связи нет - не мигаем
    }
    tracer_data_request();        // Запрос текущих параметров контроллера
    digitalWrite(2, true);
  }

  if ( millis() - previousMillis >= send_interval || flag_1Send == true ) {
    previousMillis = millis();
    if (flag_1Send) {
      flag_1Send = false;         // Снимаем флаг первой отправки
    }
    data_send();                  // Отправка данных на сервер
  }
  
  yield();
}
